@@ -1,7 +1,5 @@
-from sympy.solvers import solve
-from sympy import Poly, Eq, Function, exp
-from sympy import Symbol
-from sympy.abc import x, y, z, a, b
+from sympy import *
+from sympy.abc import x, y
 import tkinter
 from tkinter import *
 from tabulate import tabulate
@@ -15,9 +13,11 @@ import time
 import sys
 import keyboard
 import matplotlib.pyplot as plt
+from mpl_toolkits import mplot3d
+import inspect
 import re
 
-operatorsDict = {"*": 2, "/": 2, "+": 1, "%": 2, '^': 4, "-": 1}
+operatorsDict = {"*": 2, "/": 2, "+": 1, "%": 2, '': 4, "-": 1}
 operators = operatorsDict.keys()
 
 def tokenize(inputExpression):
@@ -360,8 +360,8 @@ def clear():
     
 def add(*args):
     return sum(args)
-def subtract(a, b):
-    return a-b
+def subtract(a, *args):
+    return a - sum(args)
 def multiply(*args):
     x = 1
     y = 0
@@ -369,13 +369,13 @@ def multiply(*args):
         x = x*args[y]
         y+=1
     return x
-def divide(a, b):
-    return a/b
-def listMultiply(list):
+def divide(a, *args):
+    return a / listMultiply(args)
+def listMultiply(List):
     x = 1
     y = 0
-    while y < len(lis):
-        x = x*lis[y]
+    while y < len(List):
+        x = x*List[y]
         y+=1
     return x
 def power(a, *args):    
@@ -493,10 +493,9 @@ def checkPrime(num):
     for i in range(2, num):
         if num % i == 0:
             x = True
-    if x == True:
+    if x:
         return False
-    else:
-        return True
+    return True
 def primeNumberPrinter(low, high):
     for num in range(low, high+1):
         if num > 1:
@@ -510,18 +509,38 @@ def perfectSquare(num):
 
     if math.trunc(root)-root==0:
         return True
-    else:
-        return False
+    return False
 def perfectSquarePrinter(low, high):
     for i in range(low, high):
-        if perfectSquare(i	) == True:
+        if perfectSquare(i) == True:
             print(i)
-def PerfectRootCheck(a, b):
+def perfectRootCheck(a, b):
     x = root(a, b)
     if x % 1 == 0:
         return True
-    else:
-        return False
+    return False
+def triangularCheck(n):
+    for i in range(n):
+        if i*(i+1)/2 == n:
+            return True
+    return False
+def triangularPrinter(low, high):
+    for i in range(low, high):
+        if triangularCheck(i) == True:
+            print(i)
+def pentagonalCheck(n):
+    i = 1
+    while True:
+        p = (3 * i * i - i) / 2
+        i += 1
+        if p >= n:
+            break
+    return p == n
+
+def pentagonalPrinter(low, high):
+    for i in range(low, high):
+        if pentagonalCheck(i) == True:
+            print(i)
 def fibonacci(n):
     x = (((((1+math.sqrt(5))/2)**n)-((((1-math.sqrt(5))/2)**n))))/math.sqrt(5)
     return round(x)
@@ -577,7 +596,11 @@ def divisCheck(num, num2):
         return True
     else:
         return False
-    
+def summate(n, a, b):
+    return sum(nthRange(n, a, b))
+def product(n, a, b):
+    return listMultiply(nthRange(n, a, b))
+
 def PercentagetoDecimal(percentage):
     return p / 100
 def PerToFrac(p):
@@ -602,6 +625,8 @@ def triangleArea(l, w):
     return (l*w)/2
 def trapeziumArea(a, b, h):
     return (a+b)/h
+def paraHeight(a, b):
+    return a/b
 def paraArea(b, h):
     return b*h
 def circleCircumference(r):
@@ -619,6 +644,13 @@ def surfaceArea(a, b, c):
     e = a*c
     f = b*c
     return 2*(d+e+f)
+def surfAreaPrism(farea, *args):
+    total = 2*farea
+    i = 0
+    while i < len(args):
+        total += args[i]
+        i += 1
+    return total
 def volume(a, b, c):
     return a*b*c
 def cylinderVolume(r, h):
@@ -695,12 +727,24 @@ def pytheoromside(a, b):
         c1 = b1 - a1
         c = math.sqrt(c1)
     return c
-def pytheorom(a, b):
+def pyTheoremAB(a, b):
     a1 = a*a
     b1 = b*b
     c1 = a1 + b1
     c = math.sqrt(c1)
     return c
+def pyTheoremCA(c, a):
+    c1 = c*c
+    a1 = a*a
+    b1 = c1 - a1
+    b = math.sqrt(b1)
+    return b
+def pyTheoremCB(c, b):
+    c1 = c*c
+    b1 = a*a
+    a1 = c1 - b1
+    a = math.sqrt(a1)
+    return a
 def cos_rule(c, b, A):
     c1 *= c
     b1 *= b
@@ -777,7 +821,7 @@ def nthRange(a, b, c):
     if int(b) > int(c):
         while int(c) < int(b)+1:
             x = a.replace("n", c)
-            y = evaluateExpression(x)
+            y = int(evaluateExpression(x))
             ls.append(y)
             c = int(c)
             c=int(c)+1
@@ -785,12 +829,12 @@ def nthRange(a, b, c):
     elif int(c) > int(b):
         while int(b) < int(c)+1:
             x = a.replace("n", b)
-            y = evaluateExpression(x)
+            y = int(evaluateExpression(x))
             ls.append(y)
             b = int(b)
             b=int(b)+1
             b = str(b)
-    return ls
+    return list(ls)
 def nthTable(a, b, c):
     ls = []
     a = str(a)
@@ -852,13 +896,55 @@ def average(*argv):
     total = np.sum(list(argv))
     length = len(argv)
     return total / length
-
 def consecutiveIntCalc(x):
     a = (x/3)-1
     b = x/3
     c = (x/3)+1
     return [a, b, c]
 
+def ascendingSort(*args):
+    argList = list(args)
+    argList.sort()
+    return argList
+def descendingSort(*args):
+    argList = list(args)
+    argList.sort(reverse=True)
+    return argList
+
+def ascendingPowers(a, *args):
+    args = list(args)
+    i = '0'
+    eq = ''
+    while int(i) < len(args):
+        eq += str(args[int(i)]) + '*(n**' + str(i) + ') +'
+        i = int(i)
+        i += 1
+        i = str(i)
+    eq += '0'
+    return nthFinder(eq, a)
+def ascendingPowersRange(a, b, *args):
+    args = list(args)
+    i = '0'
+    eq = ''
+    while int(i) < len(args):
+        eq += str(args[int(i)]) + '*(n**' + str(i) + ') +'
+        i = int(i)
+        i += 1
+        i = str(i)
+    eq += '0'  
+    return nthRange(eq, a, b)
+def ascendingpowersTable(a, b, *args):
+    args = list(args)
+    i = '0'
+    eq = ''
+    while int(i) < len(args):
+        eq += str(args[int(i)]) + '*(n**' + str(i) + ') +'
+        i = int(i)
+        i += 1
+        i = str(i)
+    eq += '0'  
+    return nthTable(eq, a, b)
+
 def F(m, a):
     return m*a
 def M(f, a):
@@ -974,7 +1060,15 @@ def graph(*args, lrangex=-10, urangex=10, lrangey=-10, urangey=10, graph_points=
             y = eval(args[i])
         else:
             pass
-        plt.plot(x,y)
+        fig = plt.figure()
+        ax = fig.add_subplot(1, 1, 1)
+        ax.spines["left"].set_position("center")
+        ax.spines["bottom"].set_position("zero")
+        ax.spines["right"].set_color("none")
+        ax.spines["top"].set_color("none")
+        ax.xaxis.set_ticks_position("bottom")
+        ax.yaxis.set_ticks_position("left")
+        plt.plot(x,y, "-b", label=args)
         i+=1
     plt.title(graph_title)
     plt.xlabel(xtitle,fontsize=20)
@@ -986,3 +1080,147 @@ def graph(*args, lrangex=-10, urangex=10, lrangey=-10, urangey=10, graph_points=
     if lrangey or urangey != False:
         plt.ylim([lrangey,urangey])
     plt.show()
+
+def isInfinite(x):
+    return mpmath.isinf(x)
+def isFinite(x):
+    return mpmath.isfinite(x)
+def isInt(x, gaussian = False): 
+    return mpmath.isint(x, gaussian)
+
+def Zeta(s):
+    return zeta(s)
+
+def graph(*args, lrangex=-10, urangex=10, lrangey=-10, urangey=10, graph_points=100, gridset=True, scaling=True, graph_title=None, xtitle=None, ytitle=None):
+    i = 0
+    x = np.linspace(lrangex, urangex, graph_points)
+    while i < len(args):
+        if type(eval(args[i])) == np.ndarray:
+            y = eval(args[i])
+        else:
+            pass
+        fig = plt.figure()
+        ax = fig.add_subplot(1, 1, 1)
+        ax.spines["left"].set_position("center")
+        ax.spines["bottom"].set_position("zero")
+        ax.spines["right"].set_color("none")
+        ax.spines["top"].set_color("none")
+        ax.xaxis.set_ticks_position("bottom")
+        ax.yaxis.set_ticks_position("left")
+        plt.plot(x,y, "-b", label=args)
+        i+=1
+    plt.title(graph_title)
+    plt.xlabel(xtitle,fontsize=20)
+    plt.ylabel(ytitle,fontsize=20)
+    plt.autoscale(enable=scaling)
+    plt.grid(gridset)
+    if lrangex or urangex != False:
+        plt.xlim([lrangex,urangex])
+    if lrangey or urangey != False:
+        plt.ylim([lrangey,urangey])
+    plt.show()
+def f(fx, x, y):
+    if type(eval(fx)) == np.ndarray:
+        return eval(fx)
+    else:
+        pass
+    
+def graph3dcontour(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, lrangez=-10, urangez=10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, cmap='binary'):
+    x = np.linspace(lrangex, urangex, graph_points)
+    y = np.linspace(lrangey, urangey, graph_points)
+    fig = plt.figure()
+    ax = plt.axes(projection='3d')
+    i = 0
+    while i < len(args):
+        X, Y = np.meshgrid(x, y)
+        Z = f(args[i], X, Y)
+        ax.contour3D(X, Y, Z, 50, cmap=cmap)
+        i += 1
+    ax.set_xlabel(lablex)
+    ax.set_ylabel(labley)
+    ax.set_zlabel(lablez)
+    ax.set_title(graph_title)
+    if lrangez != False or urangez == False:
+        ax.set_zlim(lrangez, urangez)
+    if lrangex != False or urangex == False:
+        ax.set_zlim(lrangex, urangex)
+    if lrangey != False or urangey == False:
+        ax.set_zlim(lrangey, urangey)
+    return plt.show()
+def linein3d(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, color='gray'):
+    x = np.linspace(lrangex, urangex, graph_points)
+    y = np.linspace(lrangey, urangey, graph_points)
+    fig = plt.figure()
+    ax = plt.axes(projection='3d')
+    i = 0
+    while i < len(args):
+        lis = args[i]
+        fz = lis[0]
+        fy = lis[1]
+        if type(eval(fz)) == np.ndarray:
+            zline = eval(fz)
+        else:
+            pass
+        if type(eval(fy)) == np.ndarray:
+            yline = eval(fy)
+        else:
+            pass
+        ax.plot3D(x, yline, zline, color)
+        i+=1
+    ax.set_xlabel(lablex)
+    ax.set_ylabel(labley)
+    ax.set_zlabel(lablez)
+    ax.set_title(graph_title)
+    if lrangez != False or urangez == False:
+        ax.set_zlim(lrangez, urangez)
+    if lrangex != False or urangex == False:
+        ax.set_zlim(lrangex, urangex)
+    if lrangey != False or urangey == False:
+        ax.set_zlim(lrangey, urangey)
+    return plt.show()
+def graph3dwire(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, lrangez=-10, urangez=10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, edgecolor=False, color='black', fill='viridis'):
+    x = np.linspace(lrangex, urangex, graph_points)
+    y = np.linspace(lrangey, urangey, graph_points)
+    ax = plt.axes(projection='3d')
+    X, Y = np.meshgrid(x, y)
+    i = 0
+    while i < len(args):
+        Z = f(args[i], X, Y)
+        if fill==False:
+            ax.plot_wireframe(X, Y, Z, color=color)
+        elif edgecolor == True:
+            ax.plot_surface(X, Y, Z, rstride=1, cstride=1,
+                    cmap=fill, edgecolor=color)
+        elif edgecolor == False:
+            ax.plot_surface(X, Y, Z, rstride=1, cstride=1,
+                    cmap=fill, edgecolor='none')
+        else:
+            raise TypeError('Parameter "edgecolor" must be a boolean operator')
+        i += 1
+    ax.set_xlabel(lablex)
+    ax.set_ylabel(labley)
+    ax.set_zlabel(lablez)
+    ax.set_title(graph_title)
+    if lrangez != False or urangez == False:
+        ax.set_zlim(lrangez, urangez)
+    if lrangex != False or urangex == False:
+        ax.set_zlim(lrangex, urangex)
+    if lrangey != False or urangey == False:
+        ax.set_zlim(lrangey, urangey)
+    return plt.show()
+
+def derivative(f_of, solvefor):
+    return diff(f_of, solvefor)
+def secondDerivative(f_of, *args):
+    return diff(f_of, args)
+def integral(f_of, solvefor):
+    integral = integrate(f_of, solvefor)
+    return integral
+def IndefIntegral(f_of, solvefor):
+    integral = str(integrate(f_of, solvefor)) + " + C"
+    return integral
+def defIntegral(f_of, solvefor, lowerbound, upperbound):
+    return integrate((f_of), (solvefor, lowerbound, upperbound))
+def Limit(f_of, solvefor, to):
+    return limit(f_of, solvefor, to)
+    
